{
        SCRSAV.PAS - A ScreenSaving unit (version 3) for TurboVision 2


        This ScreenSaving-unit for TurboVision 2.0 is (c) by Edwin Groothuis
        You are allowed to use this unit in your programs if you agree to
        these two rules:

        1. You give me proper credit in the documentation of the program.
        2. You tell me you're using this unit.

        If you don't agree with these two rules, you're not allowed to
        use it!

        How to use this ScreenSaving-unit:

        Initialisation:

        - Add the ScrSav-unit to the uses-command.
        - Make the (global) variable ScreenSaver as PScreenSaver
        - In the Initialisation of an Application, put the following
          line behind the Inherited Init:
            Inherited Init;
            ScreenSaver:=New(PScreenSaver,
                         Init(New(PWormScreenSaver,Init),6));

        Heartbeat:
        To tell the ScreenSaver that the user isn't idle at this moment,
        put the following line in the Application^.GetEvent:
          Inherited GetEvent(E);
          if E.What<>evNothing then
            if ScreenSaver<>nil then
              if E.What=evKeyDown then
              begin
                if ScreenSaver^.Saving then
                  E.What:=evNothing;
                ScreenSaver^.HeartBeat;
              end else
                if E.What and evMouse<>0 then
                  ScreenSaver^.HeartBeat;

        CountDown:
        To let the ScreenSaver know the user is idle at this moment, put
        the following line in the Application^.Idle:
          Inherited Idle;
          if ScreenSaver<>nil then
            ScreenSaver^.CountDown;

        Now start your application. After 6 seconds your screen will
        go blank. There are several ScreenSavers designed by me, if
        you have created more, I would like to have a copy of them ;-)

        A small note about the use of a delay in the ScreenSaver^.Update:
        It's not nice to use the Delay-function of the Crt-unit. Instead
        of using that, you'd better check if a certain time (100 ms, 200 ms
        and so on) has elapsed. See the StarFieldScreenSaver.Update-function
        for an example of it.

        Send all your suggestions/money/cards (I love to get cards from
        people who are using my programs) to:

        Edwin Groothuis                ECA-BBS
        Johann Strausslaan 1           tel. +31-40-550352 (up to 14k4/V42b)
        5583ZA Aalst-Waalre            FTN: 2:284/205@fidonet
        The Netherlands                     115:3145/102@pascal-net
                                       request SCRSAV for the last version!
}


unit      ScrSav;

interface

uses      Views,Objects;

type      PStandardScreenSaver=^TStandardScreenSaver;
          TStandardScreenSaver=object(TView)
                                 constructor Init;
                                 procedure Draw;virtual;
                               end;

          PMovingStarScreenSaver=^TMovingStarScreenSaver;
          TMovingStarScreenSaver=object(TView)
                                   constructor Init;
                                   procedure Draw;virtual;
                                 private
                                   dx,dy,x,y:array[1..10] of integer;
                                   LastUpdate:longint;
                                 end;


          PWormScreenSaver=^TWormScreenSaver;
          TWormScreenSaver=object(TView)
                             constructor Init;
                             procedure Draw;virtual;
                           private
                             x,y:array[1..10] of integer;
                             states:array[1..10] of char;
                             dx,dy:integer;
                             LastUpdate:longint;
                           end;

          PStarFieldScreenSaver=^TStarFieldScreenSaver;
          TStarFieldScreenSaver=object(TView)
                                   constructor Init;
                                   procedure Draw;virtual;
                                 private
                                   states:array[1..7] of char;
                                   starstate,x,y:array[1..10] of integer;
                                   LastUpdate:longint;
                                 end;

type      PScreenSaver=^TScreenSaver;
          TScreenSaver=object
                         constructor Init(_W:PView;Time:word);
                         procedure Activate;
                         procedure Deactivate;
                         procedure HeartBeat;
                         procedure CountDown;
                         procedure Update;virtual;
                         function  Saving:boolean;
                       private
                         W:PView;
                         LastBeat:longint;
                         _Saving:boolean;
                         SavedScreen:pointer;
                         SavingTime:word;
                         CursorVisible:boolean;
                       end;

implementation

uses      Drivers,App;

{----------------------------------------------------------------------------}
{----------------------------------------------------------------------------}
{ Initialise the ScreenSaver.
  Notes:
    *18.2 is because the timertick goes 18.2 times/second                    }
constructor TScreenSaver.Init(_W:PView;Time:word);
begin
  _Saving:=false;
  SavingTime:=round(Time*18.2);
  HeartBeat;
  W:=_W;
end;

{----------------------------------------------------------------------------}
{ Activate the ScreenSaver
  First, allocate the memory for the Screen.
  Second, copy the contents of the screen to the allocated memory.           }
procedure TScreenSaver.Activate;
{var       R:TRect;}
begin
  _Saving:=true;
  GetMem(SavedScreen,ScreenWidth*ScreenHeight*2);
  Move(ScreenBuffer^,SavedScreen^,ScreenWidth*ScreenHeight*2);
{  Application^.TopView^.HideCursor;
  Application^.GetExtent(R);        }
  if W<>nil then
    Desktop^.Insert(W);
  Update;
end;

{----------------------------------------------------------------------------}
{ Deactivate the ScreenSaver.
  First, copy the contents of the SavedScreen to the ScreenBuffer.
  Second, dispose the memory allocated
  Third, give the application a Redraw                                       }
procedure TScreenSaver.Deactivate;
begin
  if W<>nil then
    Desktop^.Delete(W);
  Move(SavedScreen^,ScreenBuffer^,ScreenWidth*ScreenHeight*2);
  FreeMem(SavedScreen,ScreenWidth*ScreenHeight*2);
  Application^.Redraw;
  _Saving:=false;
end;

{----------------------------------------------------------------------------}
{ The use is doing something, so stop the CountDown
  First, deactivate the ScreenSaver if Saving
  Second, update the timer                                                   }
procedure TScreenSaver.HeartBeat;
var       TT:longint absolute $40:$6c;
begin
  if Saving then Deactivate;
  LastBeat:=TT;
end;

{----------------------------------------------------------------------------}
{ CountDown to the SavingTime
  If not yet saving, look if it's time to save. If saving, update the screen }
procedure TScreenSaver.CountDown;
var       TT:longint absolute $40:$6c;
begin
  if not Saving then
  begin
    if (TT-LastBeat>SavingTime) then Activate;
  end else begin
    Update;
  end;
end;

{----------------------------------------------------------------------------}
{ Update
  Update the ScreenSaving-procedure. Override this one if you want a custom
  ScreenSaver                                                                }
procedure TScreenSaver.Update;
begin
  if W<>nil then
    W^.Draw;
end;

{----------------------------------------------------------------------------}
{ Saving
  Returns true if the Screen is being Saved.                                 }
function  TScreenSaver.Saving:boolean;
begin
  Saving:=_Saving;
end;


{----------------------------------------------------------------------------}
constructor TStandardScreenSaver.Init;
var         R:TRect;
begin
  Application^.GetExtent(R);
  Inherited Init(R);
end;

{----------------------------------------------------------------------------}
procedure TStandardScreenSaver.Draw;
begin
  ClearScreen;
end;


{----------------------------------------------------------------------------}
{ The rest are examples of different ScreenSavers.                           }
{----------------------------------------------------------------------------}
{----------------------------------------------------------------------------}
constructor TMovingStarScreenSaver.Init;
var         i:byte;
            R:TRect;
begin
  Desktop^.GetExtent(R);
  Inherited Init(R);
  Randomize;
  for i:=1 to 10 do
  begin
    x[i]:=random(ScreenWidth div 2)+(ScreenWidth div 4);
    y[i]:=random(ScreenHeight div 2)+(ScreenHeight div 4);
    dx[i]:=random(2);if dx[i]=0 then dx[i]:=-1;
    dy[i]:=random(2);if dy[i]=0 then dy[i]:=-1;
  end;
end;

{----------------------------------------------------------------------------}
procedure TMovingStarScreenSaver.Draw;
var       i:byte;
          TT:longint absolute $40:$6c;
          B:TDrawBuffer;
begin
  if TT-LastUpdate>2 then
  begin
    LastUpdate:=TT;
    ClearScreen;
    for i:=1 to 10 do
    begin
      if x[i] in [0,ScreenWidth-3]  then dx[i]:=-dx[i];
      if y[i] in [0,ScreenHeight-3] then dy[i]:=-dy[i];
      inc(x[i],dx[i]);inc(y[i],dy[i]);
      MoveChar(B,'*',i,1);
      WriteLine(x[i],y[i],1,1,B);
    end;
  end;
end;

{----------------------------------------------------------------------------}
{----------------------------------------------------------------------------}
constructor TWormScreenSaver.Init;
var         i:byte;
            xx,yy:byte;
            R:TRect;
begin
  Application^.GetExtent(R);
  Inherited Init(R);
  States[10]:=' ';
  States[9]:='';States[8]:='';
  States[7]:='';States[6]:='';
  States[5]:='';States[4]:='';
  States[3]:='';States[2]:='';

  Randomize;
  xx:=random(ScreenWidth div 2)+(ScreenWidth div 4);
  yy:=random(ScreenHeight div 2)+(ScreenHeight div 4);
  for i:=1 to 10 do
  begin
    x[i]:=xx;
    y[i]:=yy;
  end;
  dx:=random(2);if dx=0 then dx:=-1;
  dy:=random(2);if dy=0 then dy:=-1;
end;

{----------------------------------------------------------------------------}
procedure TWormScreenSaver.Draw;
var       i:byte;
          TT:longint absolute $40:$6c;
          B:TDrawBuffer;
begin
  if TT-LastUpdate>2 then
  begin
    LastUpdate:=TT;
    ClearScreen;
    if x[1]<1              then begin dx:=-dx;dy:=-random(3)+1;end;
    if x[1]>ScreenWidth-4  then begin dx:=-dx;dy:=-random(3)+1;end;
    if y[1]<1              then begin dy:=-dy;dx:=-random(3)+1; end;
    if y[1]>ScreenHeight-4 then begin dy:=-dy;dx:=-random(3)+1;end;
    for i:=10 downto 2 do
    begin
      MoveChar(B,States[i],9,1);
      WriteLine(x[i],y[i],1,1,B);
      x[i]:=x[i-1];
      y[i]:=y[i-1];
    end;
    inc(x[1],dx);inc(y[1],dy);
  end;
end;


{----------------------------------------------------------------------------}
{----------------------------------------------------------------------------}
constructor TStarFieldScreenSaver.Init;
var         i:byte;
            R:TRect;
begin
  Application^.GetExtent(R);
  Inherited Init(R);
  Randomize;
  States[1]:='';States[2]:='';States[3]:='';
  States[4]:='o';States[5]:='*';States[6]:='';States[7]:=' ';
  for i:=1 to 10 do
  begin
    x[i]:=random(ScreenWidth-1)+2;
    y[i]:=random(ScreenHeight-1)+2;
    starstate[i]:=random(7)+1;
  end;
end;

{----------------------------------------------------------------------------}
procedure TStarFieldScreenSaver.Draw;
var       i:byte;
          TT:longint absolute $40:$6c;
          B:TDrawBuffer;
begin
  if TT-LastUpdate>2 then
  begin
    LastUpdate:=TT;
    ClearScreen;
    for i:=1 to 10 do
    begin
      MoveChar(B,States[StarState[i]],i,1);
      WriteLine(x[i],y[i],1,1,B);
      StarState[i]:=(StarState[i] mod 7)+1;
      if StarState[i]=1 then
      begin
        x[i]:=random(ScreenWidth)+1;
        y[i]:=random(ScreenHeight)+1;
      end;
    end;
  end;
end;

{----------------------------------------------------------------------------}

end.